/*
    This file is part of COMMON AVR UTILS.

    COMMON AVR UTILS is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License (LGPL)
    as published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    COMMON AVR UTILS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License at <http://www.gnu.org/licenses/>
    for more details.

    (C) Georg-Johann Lay <georgjohann@web.de>
*/

#include <avr/interrupt.h>

#include "fifo.h"

void fifo_init (fifo_t * const f, uint8_t * const buf, const uint8_t size)
{
    f->pread = f->pwrite = buf;
    f->count = 0;
    f->read2end = f->write2end = f->size = size;
}

void fifo_put (fifo_t * const b, const uint8_t data)
{
    if (b->count >= b->size)
        return;
        
    uint8_t * pwrite = b->pwrite;
    
    *(pwrite++) = data;
    
    uint8_t write2end = b->write2end;
    
    if (--write2end == 0)
    {
        write2end = b->size;
        pwrite -= write2end;
    }
    
    b->write2end = write2end;
    b->pwrite = pwrite;
    
    uint8_t sreg = SREG;
    cli();
    b->count++;
    SREG = sreg;
}

#ifdef FIFO2

uint16_t fifo_get2_nocli (fifo_t * const b)
{
    return _inline_fifo_get2_nocli (b);
}

#endif // FIFO2

int fifo_get (fifo_t * const b)
{
    return _inline_fifo_get (b);
}
