/*
    This file is part of COMMON AVR UTILS.

    COMMON AVR UTILS is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License (LGPL)
    as published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    COMMON AVR UTILS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License at <http://www.gnu.org/licenses/>
    for more details.

    (C) Georg-Johann Lay <georgjohann@web.de>
*/

#ifndef FIFO_H
#define FIFO_H

#include <avr/io.h>
#include <avr/interrupt.h>

typedef struct
{
    uint8_t volatile count;
    uint8_t size;
    uint8_t *pread;
    uint8_t *pwrite;
    uint8_t read2end, write2end;
} fifo_t;

extern void fifo_init (fifo_t * const, uint8_t * const, const uint8_t);
extern void fifo_put  (fifo_t * const, const uint8_t);
extern int  fifo_get  (fifo_t * const);

static inline void 
_inline_fifo_put_nocli (fifo_t * const b, const uint8_t data)
{
//    uint8_t const size = b->size;
    
    if (b->count >= b->size)
        return;
        
    uint8_t * pwrite = b->pwrite;
    
    *(pwrite++) = data;
    
    uint8_t write2end = b->write2end;
    
    if (--write2end == 0)
    {
        write2end = b->size;
        pwrite -= write2end;
    }
    
    b->write2end = write2end;
    b->pwrite = pwrite;
    
    b->count++;
}


static inline int
_inline_fifo_get (fifo_t * const b)
{
    if (!b->count)
        return -1;
        
    uint8_t *pread = b->pread;
    uint8_t data = *(pread++);
    
    uint8_t read2end = b->read2end;
    
    if (--read2end == 0)
    {
        read2end    = b->size;
        pread -= read2end;
    }
    
    b->pread = pread;
    b->read2end = read2end;
    
    uint8_t sreg = SREG;
    cli();
    b->count--;
    SREG = sreg;
    
    return data;
}

static inline int
_inline_fifo_get_nocli (fifo_t * const b)
{
    if (!b->count)
        return -1;
        
    uint8_t *pread = b->pread;
    uint8_t data = *(pread++);
    
    uint8_t read2end = b->read2end;
    
    if (--read2end == 0)
    {
        read2end    = b->size;
        pread -= read2end;
    }
    
    b->pread = pread;
    b->read2end = read2end;
    
    b->count--;
    
    return data;
}

#ifdef FIFO2

static inline void fifo_put2 (fifo_t * const, const uint8_t, const uint8_t);

extern        uint16_t fifo_get2_nocli (fifo_t * const b);

void fifo_put2 (fifo_t * const b, const uint8_t data1, const uint8_t data2)
{
    if (b->count > (uint8_t) (b->size-2))
        return;
        
    uint8_t * pwrite = b->pwrite;
    
    *(pwrite++) = data1;
    *(pwrite++) = data2;
    
    uint8_t write2end = b->write2end;
    
    write2end -= 2;
    if (write2end == 0)
    {
        write2end = b->size;
        pwrite -= write2end;
    }
    
    b->write2end = write2end;
    b->pwrite = pwrite;
    
    uint8_t sreg = SREG;
    cli();
    b->count += 2;
    SREG = sreg;
}

static inline uint16_t _inline_fifo_get2_nocli (fifo_t * const b)
{
    uint8_t *pread = b->pread;
    
    union
    {
        struct { uint8_t lo8, hi8; };
        uint16_t u16;
    } data;
    
    data.lo8 = *(pread++);
    data.hi8 = *(pread++);
    
    uint8_t read2end = b->read2end;
    
    read2end -= 2;
    if (read2end == 0)
    {
        read2end = b->size;
        pread -= read2end;
    }
    
    b->pread = pread;
    b->read2end = read2end;
    
    b->count -= 2;
    
    return data.u16;
}

#endif // FIFO2

#endif /* FIFO_H */
