#ifndef RC5_PD_H
#define RC5_PD_H

#include <stdint.h>
#include "ports.h"

extern volatile uint16_t rc5_frame;

#ifndef RC5CYC
#error Define RC5CYC in {256, 512, 1024}
#endif // RC5CYC

#define RC5TIME     1.778e-3        // 1.778msec

#define PULSE_MIN   (uint8_t) (F_CPU / RC5CYC * RC5TIME * 0.4 + 0.5)
#define PULSE_1_2   (uint8_t) (F_CPU / RC5CYC * RC5TIME * 0.8 + 0.5)
#define PULSE_MAX   (uint8_t) (F_CPU / RC5CYC * RC5TIME * 1.2 + 0.5)

#if !defined (RC5_OWN_TIMER)

static inline void 
rc5_poll (void)
{
    static uint8_t     rc5_bit;                // bit value
    static uint8_t     rc5_time;               // count bit time
    static uint16_t    rc5_tmp;                // shift bits in

    uint16_t tmp = rc5_tmp;               // for faster access

    if (++rc5_time > PULSE_MAX)
    {         // count pulse time
        if (!(tmp & 0x4000) && (tmp & 0x2000))   // only if 14 bits received
        {
            if (rc5_frame >= 0x8000)
                rc5_frame = tmp;
        }
        tmp = 0;
    }

    if ((rc5_bit ^ N_TO_PIN (PORT_IR)) & ID_TO_BIT (PORT_IR))      // change detect
    {
        rc5_bit = ~rc5_bit;             // 0x00 -> 0xFF -> 0x00

        if (rc5_time < PULSE_MIN)          // too short
            tmp = 0;

        if (!tmp || rc5_time > PULSE_1_2)     // start or long pulse time
        {
            if (!(tmp & 0x4000))         // not to many bits
                tmp <<= 1;              // shift
            if (!(rc5_bit & ID_TO_BIT (PORT_IR)))        // inverted bit
                tmp |= 1;               // insert new bit
            rc5_time = 0;             // count next pulse time
        }
    }

    rc5_tmp = tmp;
}

#endif

typedef union
{
    struct
    {
        // The RC5 code (command)
        uint8_t code;
    
        // The device address: 0=TV ...
        uint8_t addr;
    
        // The flip bit
        volatile char flip;
    
        // This is not used by the RC5 software. It may be used by the application.
        // The main purpose of this field is to make `rc5_t' to have a size
        // of 4 bytes (instead of 3). This is much better if the user makes a local
        // copy of the rc5 object because the copy then can be hold in registers
        // and saves the overhead of a frame.
        char info;
    };
    
    uint16_t raw;
} rc5_t;

extern rc5_t rc5;

#define RC5_ALL 0xff

extern void rc5_init (uint8_t addr);
extern uint8_t rc5_decode (void);

#endif // RC5_PD_H
