#include <string.h> // memset

#include "asteroids.h"

#include "compat/interrupt.h"
#include "compat/pgmspace.h"
#include "attrib.h"

#include "frame.h"
#include "rc5-codes.h"
#include "parith-15.h"

#include "countdown.h"

// Ein Schuss
static const bh_polypoint_t laser[] PROGMEM = 
{
    {{ 0, 0}, BL_SKIP },
    {{ 5, 0}, BL_DRAW_START | BL_DRAW_END | BL_LIST_END }
};

// Das Schiff
static const bh_polypoint_t ship[] PROGMEM = 
{
    // Feuer bei Antrieb
    {{ -5, -2}, BL_SKIP },
    {{ -7,  0}, BL_DRAW_START },
    {{ -5,  2}, BL_DRAW_START | BL_DRAW_END },
    // eigentliches Raumschiff
    {{ -9, 6},  BL_SKIP },
    {{  9, 0},  BL_DRAW_START },
    {{ -9, -6}, BL_DRAW_START | BL_DRAW_END },
    {{ -3, -4}, BL_SKIP },
    {{ -3,  4}, BL_LIST_END }
};

#define R 3
// Fetter Asteriod, Radius ca. R*5.5
static const bh_polypoint_t astro4[] PROGMEM = 
{
      {{  0*R,   5*R}, BL_SKIP }
    , {{ -5*R,   2*R}, BL_DRAW_END   }
    , {{ -4*R,  -1*R}, BL_DRAW_END   }
    , {{ -3*R,  -1*R}, BL_DRAW_END   }
    , {{ -3*R,  -4*R}, BL_DRAW_END   }
    , {{  1*R,  -3*R}, BL_DRAW_END   }
    , {{  2*R,  -5*R}, BL_DRAW_END   }
    , {{  5*R,  -4*R}, BL_DRAW_END   }
    , {{  6*R,   0*R}, BL_DRAW_END   }
    , {{  4*R,   1*R}, BL_DRAW_END   }
    , {{  4*R,   3*R}, BL_DRAW_END   }
    , {{  0*R,   5*R}, BL_DRAW_END | BL_LIST_END }
};
#undef R

//#define A ({asm volatile (""::); & framebuf.asteroids.asteroids;})
#define A ({& framebuf.asteroids.asteroids;})

enum 
{
    AST_SHIP_FIRE,
    AST_SHIP,
    AST_LASER,
    AST_ASTRO4
};

static const bh_polypoint_t * asteroids_sprites[] PROGMEM =
{
    [AST_SHIP_FIRE] = ship,
    [AST_SHIP]      = ship+3,
    [AST_LASER]     = laser,
    [AST_ASTRO4]    = astro4
};

#define ADD8_SAT(X,Y)          \
 asm ("add  %0, %1"     "\n\t" \
      "brvc 0f"         "\n\t" \
      "ldi  %0, 0x7f"   "\n\t" \
      "sbrc %1, 7"      "\n\t" \
      "ldi  %0, 0x81"   "\n\t" \
      "0:"                     \
      : "+d" (X) : "r" (Y))
      
#define SUB8_SAT(X,Y)          \
 asm ("sub  %0, %1"     "\n\t" \
      "brvc 0f"         "\n\t" \
      "ldi  %0, 0x7f"   "\n\t" \
      "sbrs %1, 7"      "\n\t" \
      "ldi  %0, 0x81"   "\n\t" \
      "0:"                     \
      : "+d" (X) : "r" (Y))
      
#define NEG8_SAT(X)            \
 asm ("neg  %0"         "\n\t" \
      "brvc 0f"         "\n\t" \
      "dec  %0"         "\n\t" \
      "0:"                     \
      : "+r" (X))

static uint8_t 
onKey_asteroids (asteroids_t * a, uint8_t code)
{
    //motion_t * motion = & a->motion[0];
    bh_sprite_t * ship = & a->sprites[0];
    
    int16_t rot = ship->rot;
    if (code == RC5_LEFT)    rot += -36;
    if (code == RC5_RIGHT)   rot += 36-720;
    if (rot < 0) rot += 720;
    ship->rot = rot;
    
    point8_t sincos = _i_get_sincos8 (rot);
    
    if (code == RC5_UP2)
    {
        sincos.x /= 16;
        sincos.y /= 16;
        
        ADD8_SAT (ship->move.x, sincos.y);
        SUB8_SAT (ship->move.y, sincos.x);
        
        count.ms10.astro.antrieb = 20;
    }
    else if (code == RC5_DOWN2 && !count.ms10.astro.laser)
    {
        uint8_t laserid = A->laserid;
        
        if (++laserid == 5)
            laserid = 0;
            
        A->laserid = laserid;
        
        bh_sprite_t * laser = A->sprites + 1+laserid;

        NEG8_SAT (sincos.x);
        laser->move.x = sincos.y;
        laser->move.y = sincos.x;
        laser->move.reach = 100;
        
        
        laser->sprite_id = AST_LASER;
        laser->pos = ship->pos;
        laser->rot = ship->rot;
        
        count.ms10.astro.laser = 20;
    }
    
    return 1;
}

static uint8_t 
onPixel_explode (asteroids_t * a)
{
    BRESENHAM_BEAM_SKIP();
    
    uint8_t diam;
    uint8_t id  = a->explode.id;
    uint8_t pix = a->explode.pix;
    
    asteroids_explode_t * xdata = & A->explode.data[id];
    
    while (1)
    {
        diam = xdata->diam;
        
        if (diam)
            break;
            
        if (++id >= ASTRO_NUM_EXPLODES)
            return 1;
            
        pix = 0;
        xdata++;
    }
    
    uint16_t rot = 90 * pix;
    point8_t sincos = _i_get_sincos8 (rot);
    uint8_t x = xdata->x + 0x80;
    uint8_t y = xdata->y + 0x80;
    
    // (x,y) += diam * sincos
    fmac8_8 (x, diam, sincos.x);
    fmac8_8 (y, diam, sincos.y);
    
    BRESENHAM_BEAM_XY (x, y);
    
    if (++pix >= 8)
    {
        pix = 0;
        if (++id >= ASTRO_NUM_EXPLODES)
            return 1;
    }
    
    a->explode.id  = id;
    a->explode.pix = pix;
    
    return 0;
}

static void 
onPixel_asteroids (asteroids_t * a, void * _ UNUSED)
{
//    a = & framebuf.asteroids.asteroids;
    uint8_t id = a->drawid;
    uint8_t first = a->drawfirst;
    uint8_t done = 0;

    FLAGS |= FRAME_LOCKED;
    
    if (id == 0)
    {
        if (first)
            bh_initpoly (& A->bpoly, A->sprites, ASTRO_NUM_SPRITES);
                         
        done = bh_drawpoly (& A->bpoly);
    }
    else if (id == 1)
    {
        if (first)
        {
            a->explode.id  = 0;
            a->explode.pix = 0;
        }
        
        done = onPixel_explode (a);
    }
    
    a->drawfirst = done;
    
    if (done)
    {
        uint8_t id = a->drawid;
        
        if (++id >= 2)
        {
            id = 0;
            FLAGS &= ~FRAME_LOCKED;
        }
        
        a->drawid = id;
    }
}

// Splitter bei Explosion eins weiter
static void 
onRedraw_explode (void)
{
    asteroids_explode_t * ex = A->explode.data;
    
    for (uint8_t i=0; i < ASTRO_NUM_EXPLODES; ex++, i++)
    {
        uint8_t diam = ex->diam;
        if (diam)
            diam++;
            
        ex->diam = diam & 0x1f;
    }
}

#define STEP8(VAR,X,DX)                     \
    do {                                    \
        uint8_t _n;                         \
        int8_t _x = (X);                    \
        uint8_t _dx = (DX);                 \
        uint8_t _inc = 1;                   \
        if (_x < 0) {_inc=-_inc; _x=-_x; }  \
        DX = _n = _dx + _x;                 \
        if (_n < _dx) VAR += _inc;          \
    } while (0)

int8_t 
prandom8 (void)
{
    uint8_t r;
    
    do
        r = prandom();
    while (r == 0 || r == 0x80);
    
    return r;
}

static void 
onRedraw_asteroids (asteroids_t * a UNUSED, void * _ UNUSED)
{
    bh_sprite_t * sprite = A->sprites;
    
    //RELOAD ("y", sprite);
    
    sprite->sprite_id = count.ms10.astro.antrieb 
                        ? AST_SHIP_FIRE 
                        : AST_SHIP;
    
    for (uint8_t i=0; i < ASTRO_NUM_SPRITES; sprite++, i++)
    {
        uint8_t steps = 1;
        
        if (sprite->sprite_id == BH_SPRITE_NIL
            && i >= 6 && i <= 15)
        {
            // irgendein Wert != 0, 0x80
            if (42 == prandom8())
            {
                uint8_t rot = prandom8();
                sprite->move.rot = (rot % 8) - 4;
                {
                    int8_t x = prandom8();
                    int8_t y = prandom8();
                    int8_t x2 = x >> 1;
                    int8_t y2 = y >> 1;
                    
                    if ((int16_t) x2*x2 + y2*y2 < (int16_t) 74*74)
                        continue;
                    sprite->pos.x = x;
                    sprite->pos.y = y;
                }
                sprite->move.x = prandom8();
                sprite->move.y = prandom8();
                sprite->sprite_id = AST_ASTRO4;
            }
            
            continue;
        }
        else if (sprite->sprite_id == AST_LASER)
        {
            uint8_t reach = sprite->move.reach;
            
            if (!reach--)
            {
                sprite->sprite_id = BH_SPRITE_NIL;
                continue;
            }
            sprite->move.reach = reach;
            steps = 3;
        }
        else if (sprite->sprite_id != 0xff)
        {
            int16_t rot = sprite->rot + sprite->move.rot;
            if (rot >= 720)   rot -= 720;
            if (rot <    0)   rot += 720;
            sprite->rot = rot;
        }
        
        do
        {
            STEP8 (sprite->pos.x, sprite->move.x, sprite->move.dx);
            STEP8 (sprite->pos.y, sprite->move.y, sprite->move.dy);
        } while (--steps);
    }
    
    bh_sprite_t * laser = A->sprites;
    
    for (uint8_t i=0; i < ASTRO_NUM_SPRITES; laser++, i++)
    {
        if (AST_LASER != laser->sprite_id)
            continue;
            
        bh_sprite_t * astro = A->sprites;
    
        for (uint8_t j=0; j < ASTRO_NUM_SPRITES; astro++, j++)
        {
            if (AST_ASTRO4 != astro->sprite_id)
                continue;
                
            uint8_t dx = laser->pos.x - astro->pos.x;   if (laser->pos.x < astro->pos.x)  dx = -dx;
            uint8_t dy = laser->pos.y - astro->pos.y;   if (laser->pos.y < astro->pos.y)  dy = -dy;
            
            // Abstand durch 2 teilen, damit er in 16 Bits passt
            uint16_t dist16 = dx*(dx>>1) + dy*(dy>>1);
            
            if (dist16 <= (int16_t) 3*3*5*6/2) // Radius ca. 5.5
            {
                astro->sprite_id = BH_SPRITE_NIL;
                laser->sprite_id = BH_SPRITE_NIL;
                // Dieser Laser ist aufgebraucht, er kann keinen weiteren
                // Asteroiden zertren
                uint8_t id = A->explode.next_id;
                asteroids_explode_t * ex = A->explode.data + id;
                ex->diam = 2;
                ex->x = laser->pos.x;
                ex->y = laser->pos.y;
                
                if (++id >= ASTRO_NUM_EXPLODES)
                    id = 0;
                
                A->explode.next_id = id;
                
                break;
            }
        }
    }
    
    onRedraw_explode ();
}

static void 
onEnter_asteroids (asteroids_t * a)
{
    memset (a, 0, sizeof (*a));
    
    // a->drawid = 0;
    a->drawfirst = 1;
    a->bpoly.linedraw = & a->bline;
    a->bpoly.sprites  = asteroids_sprites;
    
    bh_sprite_t * sprite = a->sprites;
    
    // Sprites unsichtbar
    for (uint8_t i=0; i < ASTRO_NUM_SPRITES; sprite++, i++)
        sprite->sprite_id = BH_SPRITE_NIL;
}

const menuitem_t menuitem_asteroids PROGMEM =
{
    .onPixel     = onPixel_asteroids,
    .onEnter     = onEnter_asteroids,
    .onKey       = onKey_asteroids,
    .onRedraw    = onRedraw_asteroids,
    .redrawOn    = REDRAW_10_MILLI_SECONDS | REDRAW_KEY,
    .bufsize     = sizeof (asteroids_frame_t),
    .data        = & framebuf.asteroids.asteroids
};

