/*
    This file is part of MORPHEUS.

    MORPHEUS is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License (LGPL)
    as published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    MORPHEUS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License at <http://www.gnu.org/licenses/>
    for more details.

    (C) Georg-Johann Lay <georgjohann@web.de>
*/

#ifndef BRESENHAM_H
#define BRESENHAM_H

#include "vektor-zeichen.h"
#include "wire.h"

typedef struct
{
    vf_spixel_t min, max;
} boundbox_t;

// Bresenham-Algorithmus um gerade Linien zu zeichnen.
#define BL_DRAW_START (1 << 0)
#define BL_DRAW_END   (1 << 1)

#define BL_LIST_END   (1 << 2)
#define BL_SKIP       (1 << 3)

typedef struct
{
    void (*beam_xy) (vf_uint_t x, vf_uint_t y);
    // Momentane Koordinaten
    vf_upixel_t p;

    // Start- und End-Koordinaten
    vf_upixel_t start, end;

    // dd. ist Diagonalschritt
    int8_t ddx, ddy;

    // pd. ist Parallelschritt
    int8_t pdx, pdy;

    // Fehlerschritte schnell, langsam
    // el ist auch Anzahl zu zeichnender Pixel
    vf_int_t es, el;

    // Fehler
    vf_int_t err;

    // Nummer des Pixels
    vf_uint_t pix;
    
    // Anzahl Pixels
    vf_uint_t npix;

    // ersten Pixel zeichnen?
    uint8_t draw_start;
} bh_linedraw_t;


#define BH_SPRITE_NIL 0xff
// Punkt in einem Polygonzug
typedef struct
{
    vf_upixel_t p;
    uint8_t flags;
} bh_polypoint_t;

typedef struct 
{
    int8_t x, y;
    int8_t rot;
    int8_t dx, dy;
    uint8_t reach;
} bh_sprite_move_t;

typedef struct
{
    uint8_t sprite_id;
    vf_spixel_t  pos;
    uint16_t     rot;
    bh_sprite_move_t move;
} bh_sprite_t;

typedef struct
{
    bh_sprite_t  * sprite;
    bh_linedraw_t  * linedraw;
    vf_upixel_t start;
    uint8_t flags;
    uint8_t read_n_points;
    const bh_polypoint_t * sprite_points;
    const bh_polypoint_t * const * sprites;
    vf_spixel_t  sprite_pos;
    point8_t sprite_sincos;
    uint8_t num_sprites;
} bh_polydraw_t;

extern void bh_initline (bh_linedraw_t*, void(*)(vf_uint_t, vf_uint_t), vf_upixel_t, vf_upixel_t, uint8_t);
extern void bh_initpoly (bh_polydraw_t*, bh_sprite_t*,uint8_t);
extern uint8_t bh_drawline (bh_linedraw_t*);
extern uint8_t bh_drawpoly (bh_polydraw_t*);

#ifdef __i386__

extern uint16_t count_pixels (char c, uint8_t zoom);
extern void boundbox (char c, uint8_t zoom, boundbox_t *bb);

#endif // __i386__

#endif // BRESENHAM_H

