/*
    This file is part of COMMON AVR UTILS.

    COMMON AVR UTILS is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License (LGPL)
    as published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    COMMON AVR UTILS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License at <http://www.gnu.org/licenses/>
    for more details.

    (C) Georg-Johann Lay <georgjohann@web.de>
*/

#ifndef COUNTDOWN_H
#define COUNTDOWN_H

#include <stdint.h>

typedef struct
{
#ifdef COUNTDOWN_US100
    // Die Kompoinente muss mindestens das erste Elemenet enthalten
    struct
    {
        // Erstes Element ist fr internen Gebrauch!
		uint8_t timer_10ms; 
        // Ab hier die bentigten 8-Bit Countdown-Zhler eintragen
    } us100;
#endif /* COUNTDOWN_US100 */

    // Die Kompoinente muss mindestens das erste Elemenet enthalten
    struct
    {
        // Erstes Element ist fr internen Gebrauch!
		uint8_t timer_1s; 
        // Ab hier die bentigten 8-Bit Countdown-Zhler eintragen
		uint8_t wait_10ms;
		uint8_t rc5;
		uint8_t blink;
        uint8_t morse;
        
        struct
        {
            uint8_t antrieb;
            uint8_t laser;
        } astro;
		
		// vermeidet Flackern wenn dcf_new_minute kurz nach
		// quarz_new_minute auftritt
		//uint8_t new_second_p;
    } ms10;

    struct
    {
        // Ab hier die bentigten 16-Bit Countdown-Zhler eintragen
    } ms10_16;

    // Die Sekunden-Timer. 
    // Die Komponente darf auch komplett leer sein
    struct
    {
        // Erstes Element ist fr internen Gebrauch!
        uint8_t timer_1m;
        // Ab hier die bentigten 8-Bit Countdown-Zhler eintragen
        uint8_t boot;
        uint8_t debug;
        uint8_t schoner;
    } sec;

    // Die Minuten-Timer. 
    // Die Komponente darf auch komplett leer sein
    struct
    {
        // Ab hier die bentigten 8-Bit Countdown-Zhler eintragen
    } min;
} countdown_t;

extern volatile countdown_t count;

static inline
void wait_10ms (const uint8_t t)
{
#ifdef __AVR__
    asm volatile (" ; BARRIER" ::: "memory");
#endif // AVR
    count.ms10.wait_10ms = 1+t;
    while (count.ms10.wait_10ms);
}

// Alle 10 ms aufrufen wenn COUNTDOWN_US100 nicht define'd ist. 
// Alle 100 s aufrufen wenn COUNTDOWN_US100 define'd ist.
extern void job_countdown (void);

#endif /* COUNTDOWN_H */
