/*
    This file is part of MORPHEUS.

    MORPHEUS is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License (LGPL)
    as published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    MORPHEUS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License at <http://www.gnu.org/licenses/>
    for more details.

    (C) Georg-Johann Lay <georgjohann@web.de>
*/

#ifndef DAC_H
#define DAC_H

#include <stdint.h>

#if !defined __AVR__

extern void dac_p (void);

#else // AVR

#include "ports.h"
#include "frame.h"
#include "pixel.h"

static inline void dac_p (void);

#define MOV_BIT(a,pa,b,pb) \
    asm ( \
        "bst %[rb],%[bb]" "\n\t" \
        "bld %[ra],%[ba]"        \
        : [ra] "+r" (a) \
        : [rb]  "r" (b), [ba] "n" (pa), [bb] "n" (pb) \
    )

void dac_p (void)
{
    uint8_t x = reg_pixel.x;
    uint8_t y = reg_pixel.y;
    
    y = ~y;
    
    uint8_t led = PORTD; // fr LED
    led |= 0b100;
    uint8_t portd = PORTD;
    uint8_t portb = PORTB;

/*    
	PORT_DAC6 = PORTB_5, // SCK
	PORT_DAC7 = PORTB_4, // MISO
    
    PORT_DAC_CS = PORTD_2, // 0=Enable
    PORT_DAC_AB = PORTD_3, // MOSI, 0=A, 1=B = LED
*/    
    portd  &= ~ 0b1100;
    
    MOV_BIT (portb,5, x,6);
    MOV_BIT (portb,4, x,7);

    // DAC-Wert fr x an PORT B+C    
    PORTC = x;
    PORTB = portb;
    
    MOV_BIT (portb,5, y,6);
    MOV_BIT (portb,4, y,7);
    
    // DAC.B laden
    PORTD = portd | (1 << 3);
    SET (PORT_DAC_CS);

    // DAC-Wert fr y an PORT B+C    
    PORTC = y;
    PORTB = portb;
    // DAC.A laden
    PORTD = portd;
    SET (PORT_DAC_CS);
    
    // Standard wieder herstellen (LED, CS=1)
    PORTD = led;
}
#endif //AVR

#endif // DAC_H
