/*
    This file is part of MORPHEUS.

    MORPHEUS is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License (LGPL)
    as published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    MORPHEUS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License at <http://www.gnu.org/licenses/>
    for more details.

    (C) Georg-Johann Lay <georgjohann@web.de>
*/

#include <string.h>

#include "compat/interrupt.h"
#include "attrib.h"

#include "frame.h"
#include "rc5-codes.h"

#include "dcf-oszi.h"

typedef struct dcfoszi_t
{
    uint8_t offy, line, err;
    uint8_t bit, bit_insert;
    uint8_t dcf_bit;
} dcfoszi_t;

dcfoszi_t dcfoszi;

#define OSZI_HEIGHT 230
#define OSZI_HUB    5

#define OSZI_X0 (0x80 - 100/2)
#define OSZI_Y0 (0x80 - OSZI_HEIGHT/2)

static uint8_t 
onKey_dcfoszi (dcfoszi_t * o, uint8_t code)
{
    if (0x80 == code || 0x81 == code)
    {
        o->dcf_bit = code;
        return 1;
    }
    
    return 0;
}

static void 
onPixel_dcfoszi (dcfoszi_t * t, void * tf UNUSED)
{
    uint8_t bit  = t->bit;
    uint8_t line = t->line;
    
    FLAGS |= FRAME_LOCKED;
    
    if (++bit >= 100)
    {
        bit = 0;
        
        if (++line >= OSZI_LINES)
        {
            line = 0;
            FLAGS &= ~FRAME_LOCKED;
        }
    }
    
    if (bit == 99)
        beam_skip();
    
    t->bit  = bit;
    t->line = line;
    
    uint8_t x = OSZI_X0 + bit;
    uint8_t y = OSZI_Y0 + line * (OSZI_HEIGHT / OSZI_LINES);
    
    uint8_t byte = framebuf.oszi.bits[line+1][bit / 8];
    bit &= 7;
    
    if (byte & (1 << bit))
        y += OSZI_HUB;
    
    beam_xy (x, y + t->offy);
}

static void 
onRedraw_dcfoszi (dcfoszi_t * o, uint8_t bits[OSZI_LINES+1][1+100/8])
{
    uint8_t sreg = SREG;
    
//    cli();
    //FLAGS |= FRAME_VALID;
    
    uint8_t bit = o->bit_insert;
    
    uint8_t mask = 1 << (bit & 7);
    uint8_t * addr = & bits[0][bit >> 3];
    
    uint8_t byte = *addr;
    byte &= ~mask;
    if (1 & o->dcf_bit)
        byte |= mask;
        
    *addr = byte;
    
    uint8_t err  = o->err;
    uint8_t offy = o->offy;
    
    err += OSZI_HEIGHT / OSZI_LINES;
    if (err >= 100)
    {
        err -= 100;
        offy++;
    }
    
        
    if (++bit >= 100)
    {
        memmove (bits[1], bits[0], OSZI_LINES * sizeof (bits[0]));
        bit = 0;
        offy = 0;
        err = 0;
    }
    
    o->err        = err;
    o->offy       = offy;
    o->bit_insert = bit;
    
    SREG = sreg;
}

static void 
onEnter_dcfoszi (dcfoszi_t * t UNUSED)
{
}

const menuitem_t menuitem_dcfoszi PROGMEM =
{
    .onPixel     = onPixel_dcfoszi,
    .onEnter     = onEnter_dcfoszi,
    .onKey       = onKey_dcfoszi,
    .onRedraw    = onRedraw_dcfoszi,
    .redrawOn    = REDRAW_DCF,
    .bufsize     = 0,
    .data        = & dcfoszi
};
