/*
    This file is part of MORPHEUS.

    MORPHEUS is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License (LGPL)
    as published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    MORPHEUS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License at <http://www.gnu.org/licenses/>
    for more details.

    (C) Georg-Johann Lay <georgjohann@web.de>
*/

#ifndef FRAME_H
#define FRAME_H

#include <stdint.h>
#include "attrib.h"
#include "flags.h"
#include "pixel.h"

#define FRAME_SCHONER_TIMEOUT 20 // Sekunden

#include "beam-funcs.h"
#include "uhr.h"
#include "snake.h"
#include "asteroids.h"
#include "text.h"
#include "dcf-oszi.h"

#include "testbild.h"

typedef union framebuf_t
{
    uhr_t uhr;
    
    struct
    {
        snake_frame_t buf[3];
    } snake;
    struct
    {
        text_frame_t buf[3];
    } text;
    struct
    {
        pixel8_t atom_xy[NATOMS];
    } test;
    struct
    {
        uint8_t bits[OSZI_LINES+1][1+100/8];
    } oszi;
    struct
    {
        asteroids_frame_t buf[3];
        asteroids_t asteroids;
    } asteroids;
} framebuf_t;

typedef struct
{
    // Austasten in Ticks
    int8_t off;
    int8_t off2;
    // Ticks in diesem Frame
    uint16_t volatile ticks;

    // Bildschirmschoner in Sekunden als Ladewert fr
    // count.sec.schoner
    uint8_t schoner_timeout;
    
    struct
    {
        void * isr;
        void * isr_new;
        void * tmp;
        void * tmp_new;
    } buffers;
} frame_t;

extern void frame_activate (frame_t*);
extern void frame_bufinit (frame_t*,uint16_t);

extern framebuf_t framebuf;
extern frame_t frame;

#endif /* FRAME_H */
