/*
    This file is part of MORPHEUS.

    MORPHEUS is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License (LGPL)
    as published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    MORPHEUS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License at <http://www.gnu.org/licenses/>
    for more details.

    (C) Georg-Johann Lay <georgjohann@web.de>
*/

#ifndef MAIN_H
#define MAIN_H

#include <stdint.h>

enum
{
    // vor main()
    BOOT_INIT,
    // Initialisierung in main()
    BOOT_SETUP,
    // Rhre aufheizen
    BOOT_HEATING,
    // Hochspannung stabilisieren
    BOOT_POWERING,
    // Heizung + Hochspannung ok, es kann losgehen! 
    BOOT_DONE
};

typedef struct
{
    // Maximum an Ticks fr Timer ISR bis zum Fertigstellen von 
    // pixel_on_the_fly()
    uint16_t ticks_max;
    uint8_t loop;
} statistic_t;

#include "attrib.h"
typedef struct
{
    volatile uint8_t debug __attribute__((deprecated));
    uint8_t bootlevel;
    uint8_t mcusr;
    statistic_t stat;
    
    // Timer-ISR
    struct
    {
        uint16_t num_10ms;
        uint8_t  num_1ms;
        uint8_t  ocr_err;
    } irq;
    
} system_t;

extern system_t sys;

#ifdef DEBUG
#   include "uput.h"
#   define DPRINT(x,y...) \
    do { if (sys.debug) { sys.debug--; put_PSTR(PSTR(x) , ##y); }} while(0)
#else
#   undef  put_P
#   define put_P(...) (void) 0
#endif // DEBUG    

#endif /* MAIN_H */
