/*
    This file is part of MORPHEUS.

    MORPHEUS is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License (LGPL)
    as published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    MORPHEUS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License at <http://www.gnu.org/licenses/>
    for more details.

    (C) Georg-Johann Lay <georgjohann@web.de>
*/
#ifndef MENU_H
#define MENU_H

#include <stdint.h>
#include "attrib.h"

extern uint8_t rc5_code;

#define REDRAW_1_SECOND             (1 << 0)
#define REDRAW_1_MILLI_SECOND       (1 << 1)
#define REDRAW_10_MILLI_SECONDS     (1 << 2)
#define REDRAW_KEY                  (1 << 3)
#define REDRAW_ENTER                (1 << 4)
#define REDRAW_DCF                  (1 << 5)

typedef struct
{
    void * onPixel;
    void * onRedraw;
    void * onKey;
    void * onEnter;
    void * data;
    uint8_t bufsize;
    uint8_t redrawOn;
} menuitem_t;

typedef struct
{
    uint8_t item_id;
    
    uint8_t sel_id;
    uint8_t item_count;
    uint8_t menu_id;
    // Was erfordert onRedraw?
    uint8_t redraw;
    int8_t dcf_bit;
    
    volatile uint8_t tick_10ms;
    volatile uint8_t tick_1ms;
    
    menuitem_t item;
    
} menu_t;

extern menu_t menu;

#define OFFSETOF(X,Y) (((int) &X.Y-((int) &X))/sizeof(X.Y))

enum
{
    ITEMID_UHR,
    ITEMID_TEXT,
    ITEMID_TEST,
    ITEMID_SNAKE,
    ITEMID_SCHONER,
    ITEMID_DCFOSZI,
    ITEMID_ASTEROIDS,
    
    ITEMIDS,
    
    ITEMID_TEXTMENU = ITEMIDS,
    ITEMID_CONFIG
};

extern void NORETURN menu_main (menu_t*);

typedef void    (*onEnter_t)       (void*);
typedef uint8_t (*onKey_t)         (void*, uint8_t);
typedef void    (*onPixel_t)       (void*, void*);
typedef void    (*onRedraw_t)      (void*, void*);

extern void menu_activate_item (menu_t*, uint8_t);

#include "vektor-zeichen.h"

extern vecfont_t vecfont;

#endif /* MENU_H */
