/*
    This file is part of MORPHEUS.

    MORPHEUS is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License (LGPL)
    as published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    MORPHEUS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License at <http://www.gnu.org/licenses/>
    for more details.

    (C) Georg-Johann Lay <georgjohann@web.de>
*/

#include <avr/io.h>
#include "attrib.h"

#include "morse.h"
#include "ports.h"

#define FREQ(x) (F_CPU/(2*(x)-1))

morse_t morse_data;

static void morse_freq_init (void);

void CONSTRUCTOR 
morse_init (void)
{
    MAKE_OUT (PORT_SPKA);
    MAKE_OUT (PORT_SPKB);
    SET (PORT_SPKA);
    SET (PORT_SPKB);

    morse_freq_init ();
}

void 
morse_freq_init (void)
{
    // Mode #8 fr Timer1 (Manual S. 97/ATmega8, S.133/ATmega168)
    // PWM, Phase & Freq correct, TOP = ICR1
    // und volle MCU-Clock
    TCCR1A = 0;
    TCCR1B = (1 << WGM13) | (1 << CS10);

    // ICR1 fr gewnschte Frequenz
    uint16_t icr1 = FREQ (930);

    ICR1 = icr1;
    // Tastverhltnis = 50%
    OCR1A = icr1/2;
    OCR1B = icr1/2;
}

void 
morse (uint8_t b)
{
    if (morse_data.bits)
        return;
        
    if (b <= 0)
    {
        morse_data.bits = 0;
        return;
    }

    uint8_t buch = (uint8_t) b;

    const uint8_t lang = buch;
    uint8_t bits = 8;

    do
    {
        bits--;
        buch <<= 1;
    }
    while (buch < 0x80);

    if (lang < 0x80)
        buch = ~buch;

    morse_data.buch = buch;
    morse_data.bits = bits;
    morse_data.stage = 2;
    
    morse_freq_init();
}

void
job_morse (morse_t * m)
{
    uint8_t bits = m->bits;

    if (0 == bits)
        return;

    uint8_t stage = m->stage;
    uint8_t wait  = m->wait;

    if (2 == stage)
    {
        uint8_t buch = m->buch;

        wait = 3 * MORSE_DOT;

        if (buch & 0x80)
            wait = MORSE_DOT;

        m->buch = buch << 1;
        speaker_on();
        stage--;
    }
    else if (1 == stage)
    {
        if (!--wait)
        {
            speaker_off();
            wait = MORSE_DOT;
            if (bits == 1)
                wait = 3 * MORSE_DOT;
            stage--;
        }
    }
    else if (0 == stage)
    {
        if (!--wait)
        {
            bits--;
            stage = 2;
        }
    }

    m->stage = stage;
    m->bits = bits;
    m->wait = wait;
}


#define BASE 440

#define C0      1.0
#define D0      1.122
#define E0      1.26
#define FIS0    1.414
#define G0      1.498
#define A0      1.682
#define H0      1.888
#define C1      2*1.0
#define D1      2*1.122
