#ifdef PANIC
#if PANIC

#include <avr/wdt.h>
#include <util/delay.h>
#include "compat/interrupt.h"

#include "panic.h"

#include "ports.h"

static void delay_10ms (uint8_t n)
{
    uint16_t n2 = n * 500;
    do
    {
        _delay_us (20);
        TOGGLE (PORT_Z);
        wdt_reset();
    }
    while (--n2);
}


static void 
morse_wait (uint8_t buch)
{
    // 0 wird nicht ausgegeben
	if (!buch)
		return;
		
    // Bit 7 sagt, ob das hchstwertige der Bits 0..6
    // fr ein DIT steht oder fr ein DAH
	const uint8_t lang = buch & 0x80;
	uint8_t bits = 8;
	
    // Das MSB in den Bits 0..6 suchen
    // 'bits' ist die 1 + die Nummer des Bits 
    // und gibt an, aus wieviel DITs+DAHs das Zeichen
    // besteht
	do
	{
		buch <<= 1;
		bits--;
    }
    // Solange schieben, bis das erste 1-Bit gefunden ist
	while (! (buch & 0x80));

    // Das MSB ist nun in Bit 7 gelandet
    // Forme 'buch' so um, da 
    // 1 = DIT
    // 0 = DAH
    // Wenn lang != 0 ist, dann codiert das MSB fr ein DAH,
    // daher mssen in dem Fall alle Bits umgedreht werden.
	if (!lang)
		buch = ~buch;
	
	do
	{
        // DIT bzw DAH (dauert 3 DITs) ausgeben
		speaker_on();
		delay_10ms ((buch & 0x80) ? MORSE_DOT : 3*MORSE_DOT);
        // Pause ist so lange wie ein DIT
		speaker_off();
		delay_10ms (MORSE_DOT);
        // nchstes Bit
		buch <<= 1;
	}
	while (--bits);
		
    // Pause zwischen 2 Zeichen ist so lange wie ein DAH
    // (1/3 der Pause wurde schon in der Schleife gewartet)
	delay_10ms (2*MORSE_DOT);
}

void __vector_default (void)
{
    panic (_Q_);
}

void panic_func (uint8_t n)
{
    cli();
    
    uint8_t panics = 3;
    
    CLR (PORT_POWER);
    
    morse_init();
    speaker_off();
    
    while (1)
    {
        if (panics)
        {
            panics--;
            morse_wait (n);
        }
        delay_10ms (100);
    }
}

#endif /* PANIC != 0*/
#endif /* PANIC */
