#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <inttypes.h>

#define printf(FMT,X...) fprintf (stderr, "@%02x: " FMT, snake.draw.elem , ##X)
#define printf(...) (void) 0

////////////////////////////////////////////////////////////
// Datenaustausch mit Snake
#include "snake.h"
#include "beam.h"

// beam.c
beam_t beam;
beambuf_t beambuf;

// menu.c
vecfont_t vecfont;
menu_t menu;
static void menu_set_item (menu_t * m, uint8_t id);

// <avr/io.h>
uint8_t volatile GPIOR0;
uint8_t volatile SREG;

// <avr/interrupt.h>
void cli (void) {}
void sei (void) {}

//struct { uint16_t ticks; } beam;

void menu_set_item (menu_t * m, uint8_t id)
{
    m->item_id = id;
    
    uint8_t sreg = SREG;
    cli();

    // Die Beam-Puffer werden ungltig
    FLAGS &= ~BEAM_VALID;
    
    menuitem_t * pitem = & menuitem_snake;
    
    memcpy_P (& m->item, pitem, sizeof (menuitem_t));
    
    // Evtl. Sperrung aufheben
    FLAGS &= ~BEAM_LOCKED;
    
    vecfont_reset (&vecfont);
    
    beam_bufinit (menu.item.bufsize);
    
    SREG = sreg;
    
    //put_P ("enter %d -> 0x%X...\n", id, 2*(int) m->item.onEnter);
    
    if (m->item.onEnter)
        ((onEnter_t) m->item.onEnter) (m->item.data);
    //put_P ("e: ok\n");
}


////////////////////////////////////////////////////////////
// Von der Kommandozeile
int level;
int frame;

////////////////////////////////////////////////////////////
// Stimuli

#include "rc5-codes.h"

#define ZZZ   0,0,0,0,0,0,0,
#define _     0,ZZZ

#define W   RC5_LEFT,  ZZZ // WEST
#define E   RC5_RIGHT, ZZZ // OST
#define N   RC5_UP,    ZZZ // NORD
#define S   RC5_DOWN,  ZZZ // SD

const uint8_t stimuli[] = 
{
    0,
    W _ _ _ _ S _ _ _ _ W _ _ _ _ N _ _ E _ _ _ S E 
    _ _ _ _ _ _ _ _ _ _ N _ _ W S _ E  
    0
};

////////////////////////////////////////////////////////////
// Grafik

#define PIX_X 256
#define PIX_Y 256

double pixel[PIX_X][PIX_Y];

// Farben
struct { int r, g, b;}
BEAM_COLOR = { 0, 15, 7 };

#define VOID_COLOR "5 3 1 "

// Pixel nach Strahlfarbe aufhellen
static void heller (vf_uint_t x, vf_uint_t y, double h)
{
    pixel[x][y] = 1.0 - (1.0 - h) * (1.0 - pixel[x][y]);
}

////////////////////////////////////////////////////////////
// Strahl-Routinen

int skip = 0;

void sexit()
{
//    fprintf (stderr, "EXIT (tick=%d)  @%02x\n", beam.ticks, snake.draw.elem);
    exit (2);
}

void beam_skip (void)
{
    printf ("\nskip\n");
    beam.ticks++;
    if (beam.ticks > 2000)
        sexit();
    skip = 1;
}

void beam_xy (vf_uint_t x, vf_uint_t y)
{
    printf ("xy %d %d\n", x, y);
    beam.ticks++;
    if (beam.ticks > 2000)
        sexit();

    // Nur zeichnen fr den angeforderten Frame
    if (frame)
        return;

    static vf_uint_t beam_x;
    static vf_uint_t beam_y;
        
    // Beim Zeichnen der Mauer wird der Strahl nachgezogen
/*    
    if (!skip && (snake.draw.elem == 0xfc || snake.draw.elem == 0xfd))
    {
        int dx = beam_x < x ? 1 : beam_x > x ? -1 : 0;
        int dy = beam_y < y ? 1 : beam_y > y ? -1 : 0;

        if (dx*dy != 0)
        {         
            printf ("[%d %d] (%d,%d) --> [%d %d]  @%02x\n",
                    x, y,  dx, dy,   beam_x, beam_y,
                    snake.draw.elem);
            
            exit (3);
        }        

        while (1)
        {
            beam_x += dx;  
            beam_y += dy;

            if (x == beam_x && y == beam_y)
                break;
            heller (beam_x, beam_y, 0.4);
        }
    }
*/
    // Regulrer Pixel
    heller (beam_x = x, beam_y = y, 0.8);

    skip = 0;
}

void vf_notify_char (vecfont_t *vf, char c)
{
    if (isalnum (c))
        printf ("NOTITY '%c'\n", c);
    else    
        printf ("NOTITY 0x%02x\n", c);
}

////////////////////////////////////////////////////////////
int main (int argc, char * argv[])
{
    ////////////////////////////////////////////////////////////
    // Kommandozeile
    if (argc == 3)
    {
        sscanf (argv[1], "%i", & level);
        sscanf (argv[2], "%i", & frame);
    }

    // Parameter-Anzahl und Wertebereiche erzwingen
    if (argc != 3
        || frame < 1
        || (level-1) & ~3)
    {
        fprintf (stderr, "\nVerwendung:\n%s <level> <frame>\n", argv[0]);
        fprintf (stderr, "    level in 1..4 anstatt %d\n", level);
        fprintf (stderr, "    frame > 0     anstatt %d\n", frame);
        exit (1);
    }

    ////////////////////////////////////////////////////////////
    vecfont.beam_skip = beam_skip;
    vecfont.beam_xy   = beam_xy;
    ////////////////////////////////////////////////////////////
    // Frame zeichnen

    // Screen lschen    
    memset (pixel, 0, sizeof (pixel));

    // Menu-Punkt betreten
    printf ("menu_set_item\n");
    menu_set_item (&menu, ITEMID_SNAKE);

//    ((onEnter_t) menuitem_snake.onEnter) (&snake);

    // Level-Parameter berschreiben
    snake_set_level (&snake, level);
    snake.level.delay = 1;
    snake.level.grow  = 20;
    
    
    ((onRedraw_t) menuitem_snake.onRedraw) (&snake, beam.buffers.tmp);
    beam_activate();
    
    // Einen Pixel im voraus
    ((onPixel_t) menuitem_snake.onPixel) (&snake, beam.buffers.isr);
    ((onPixel_t) menuitem_snake.onPixel) (&snake, beam.buffers.isr);
    printf ("A: FLAGS = %x\n",  FLAGS);

    // Index in Stimuli-Vektor
    int stim = 0;

    //((onPixel_t) menuitem_snake.onPixel) (&snake, beam.buffers.isr);
    // Frames erzeugen
    do 
    {
        snake.draw_done = 0;
        frame--;
        // Solange Pixeln bis der aktuelle Frame komplett ist
        do
        {
            ((onPixel_t) menuitem_snake.onPixel) (&snake, beam.buffers.isr);
        } while (!snake.draw_done);

        // Stimulus senden
        if (stim++ < sizeof (stimuli) / sizeof (stimuli[0])) 
            if (stimuli[stim-1])
                ((onKey_t) menuitem_snake.onKey) (&snake, stimuli[stim-1]);

        // tempus fugit
        ((onRedraw_t) menuitem_snake.onRedraw) (&snake, beam.buffers.tmp);
    } while (frame);

    ////////////////////////////////////////////////////////////
    // Grafik als .ppm auf stdout ausgeben
    
#undef printf
    //exit(0);
    printf ("P3\n%d %d\n15\n", PIX_X, PIX_Y);

#define R 126
#define M 128

    for (int y = PIX_Y-1; y >= 0; y--)
    {
        for (int x = 0; x < PIX_X; x++)
        {
            // Pythagoras
            if ((x-M)*(x-M) + (y-M)*(y-M) < R*R)
            {
                double v = pixel[x][y];
                printf ("%d %d %d ",
                        (int) (v * BEAM_COLOR.r), 
                        (int) (v * BEAM_COLOR.g),
                        (int) (v * BEAM_COLOR.b)
                );
            }
            else 
                printf (VOID_COLOR);
        }

        printf ("\n");
    }

    ////////////////////////////////////////////////////////////
    // Fertig!
    return 0;
}
