/*
    This file is part of MORPHEUS.

    MORPHEUS is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License (LGPL)
    as published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    MORPHEUS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License at <http://www.gnu.org/licenses/>
    for more details.

    (C) Georg-Johann Lay <georgjohann@web.de>
*/

#include "compat/pgmspace.h"
#include "compat/interrupt.h"
#include <string.h>

#include "attrib.h"
#include "rc5-codes.h"
#include "frame.h"
#include "text.h"
#include "time.h"

#ifdef DEBUG
#   include "uput.h"
#   define sprintf(STR,FMT,X...)    put_P ("%@" FMT, STR , ##X)
#else
#   define sprintf(STR,FMT,X...)    strcpy_P (STR, PSTR(FMT))
#endif // DEBUG

text_t text = 
{
    .zoom = 2,
    .textid = 1,
};

static const char text1[] PROGMEM =
        "0123456789.:\n"
        "ABC\b_D\b_EFGHIJK\n"
        "LMNOPQRSTUV\n"
        "WXYZ\n"
        "abcdefghijklmn\n"
        "opqrstuvwxyz\n"
        "+-*?!,;<>^\\_#=@"
        ;

static const char text2[] PROGMEM =
    "Franz jagt im\n"
    "komplett ver-\n"
    "wahrlosten Taxi\n"
    "quer durch\n"
    "Bayern.$~~'"
    ;
        
/*
void CONSTRUCTOR _init_text (void)
{
    memset (text4, VF_UNKNOWN, TEXT4_LEN-1);
}
*/

static uint8_t 
onKey_text (text_t * t, uint8_t code)
{
    uint8_t redraw = 1;
    
    uint8_t zoom = t->zoom;
    
    if (code == RC5_LEFT)  zoom--;
    if (code == RC5_RIGHT) zoom++;
    
    if (zoom > 0 && zoom < 15)
        t->zoom = zoom;
        
    if (code == 10)
        t->proportional = ! t->proportional;
    
    uint8_t textid = t->textid;
    
    if (code == RC5_ROT)           textid = 1;
    else if (code == RC5_GRUN)     textid = 2;
    else if (code == RC5_GELB)     textid = 3;
    else if (code == RC5_BLAU)     textid = 4;
    t->textid = textid;
/*    
    if ((code & 0x80) && textid == 4)
    {
        uint8_t c = code & ~0x80;
        if (c && c != '\r')
        {
            put_P ("%@%s%c", text4, text4+1, c);
            text4[TEXT4_LEN-1] = 0;
        }
            
    }
    
    text_set_text (t, textid);
*/    
    uint8_t ret = redraw;
    return ret;
}

// Helfer fr 'onPixel_text'
// Initialisiert 'vecfont' auf den Text der Nummer 'id'
// aus dem Puffer 'tb'. Es werden nur Texte bercksichtigt
// bzw. gezhlt, die nicht leer sind (tb->text.pstr != 0).
// Falls keinen solchen Text gibt, wird BEAM_LOCKED = 0
// gesetzt, d.h. alle Texte sind angezeigt worden, und im
// nchsten Durchlauf von 'onPixel_text' wird wieder mit dem
// ersten Text begonnen.
static void 
init_vecfont (uint8_t idx, text_frame_t * tb)
{
    // Zhlt nicht-leere Texte
    uint8_t id_nz = 0;
    
    // Zhlt alle Texte
    uint8_t id = 0;
    
    // Zeiger auf den zu untersuchenden Text
    text_t * t = & tb->text[0];
    
    while (1)
    {
        if (t->pstr)
        {
            if (id_nz == idx)
            {
                // Text gefunden: Info in Font-Info eintragen
                vecfont_t *vf = &vecfont;
                
                vf->text         = t->pstr;
                vf->proportional = t->proportional;
                vf->zoom         = t->zoom;
                vf->text_in_flash= t->text_in_flash;
                vf->start        = t->start;
                // vecfont_reset (vf);
                return;
            }
            
            // Text-No stimmt noch nicht: eins weiter
            id_nz++;
        }
        
        t++;
        if (++id >= sizeof (tb->text) / sizeof (tb->text[0]))
        {
            // Es gibt keine Texte mehr; alle wurden angezeigt.
            // Lock aufheben.
            FLAGS &= ~FRAME_LOCKED;
            return;
        }
    }
}

void
onPixel_text (text_t * t UNUSED, text_frame_t * tb)
{
    static uint8_t text_no;
    
    if (!IS_FLAG (FRAME_LOCKED))
    {
        // Soeben wurde der Puffer (mglicherweise)
        // aktualisiert und/oder der letzte Durchlauf der
        // Textanzeige war fertig.
        // Anzeige-Zyklus beginnt von neuem.
        asm volatile (""::);
        init_vecfont (text_no = 0, tb);
    }
    
    // Strahl LOCKen bis alle Texte angezeigt sind.
    // Das Entsperren geschieht in 'init_vecfont'.
    FLAGS |= FRAME_LOCKED;
    
    if (vecfont_draw_letters (& vecfont))
        init_vecfont (++text_no, tb);
}

static void onEnter_text (text_t * t UNUSED)
{
    vecfont_reset (& vecfont);
}

#include "countdown.h"

static void 
onRedraw_text (text_t * t, text_frame_t * tb)
{
    // Leer
    tb->text[1].pstr = (const char*) 0;
    // #0 enthlt den Text
    text_t * t0 = & tb->text[0];
    
    uint8_t id   = t->textid;
    uint8_t zoom = t->zoom;
    
    t0->text_in_flash = (id <= 2);
    t0->proportional  = t->proportional;
    t0->zoom          = zoom;
    
    const char * text = (void*) 0;
    
    upixel8_t start = {40, 180};    

    if (1 == id)
    {
        text = text1;
    }
    else if (2 == id)
    {
        text = text2;
    }
    else if (3 == id)
    {

        text = tb->str;
        start = (upixel8_t) {40+2*zoom, 180-4*zoom};
        sprintf (tb->str, "%2:%2:%2.%2", time.hour, time.minute, time.second, 100-count.ms10.timer_1s);
    }
    else if (4 == id)
    {
        text = tb->str;
        sprintf (tb->str, "?????????");
    }
    
    t0->start = start;
    t0->pstr  = text;
}

const menuitem_t menuitem_text PROGMEM = 
{
    .onPixel        = onPixel_text,
    .onRedraw       = onRedraw_text,
    .onEnter        = onEnter_text,
    .onKey          = onKey_text,
    .redrawOn       = REDRAW_10_MILLI_SECONDS,
    .bufsize        = sizeof (text_frame_t),
    .data           = & text
};
