/*
    This file is part of MORPHEUS.

    MORPHEUS is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License (LGPL)
    as published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    MORPHEUS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License at <http://www.gnu.org/licenses/>
    for more details.

    (C) Georg-Johann Lay <georgjohann@web.de>
*/

#ifndef WIRE_H
#define WIRE_H

#include <stdint.h>

#include "sintab.h"

#include "data-types.h"

typedef int8_t  frac8_t;
typedef int16_t frac16_t;
typedef int32_t frac32_t;

extern void wire1 (void);
extern void wire2 (void);
extern void wiref (void);
extern void wiremac (void);

extern frac16_t frac_a;
extern frac16_t frac_b;
extern frac16_t frac_c;

//#define REG_SIZE "2" // 2, 3, 4, 5
#define TMP16 "2"    // 2, 3
#define ZERO  "4"
//#define TMP8  "3"

typedef union
{
    struct { frac8_t x, y; };
    uint16_t xy;
} point8_t;

typedef union
{
    struct { frac16_t x, y; };
    struct { int8_t :8, x, :8, y; } hi;
    uint32_t xy;
} point16_t;

//register int32_t reg_size  asm (REG_SIZE) __attribute__((deprecated));
register uint16_t reg_tmp16 asm (TMP16);
register uint8_t reg_zero  asm (ZERO);
//register uint8_t reg_tmp8  asm (TMP8);

//register frac16_t reg_ty asm (REG_TY);


typedef struct
{
    frac32_t x, y;
} point32_t;

typedef struct
{
    frac16_t sin, cos;
} angle16_t;

/*
typedef struct
{
    point16_t p;
    frac16_t dp;
    frac16_t radius;
    int16_t phi;
    int32_t dphi;
} wire_t;

typedef union
{
    int32_t s32;

    struct
    {
        frac16_t low;
        frac16_t frac16;
    } data16;

    struct
    {
        uint8_t low;
        frac16_t frac16;
        uint8_t high;
    } data24;

    frac32_t frac32;
} wdata32_t;
*/
/*
typedef struct
{
    point16_t p;
    frac16_t dp;
    frac16_t radius;
    int32_t phi; // phi << 16
    int32_t dphi;
    uint16_t index;
} wdata_t;
*/
/*
#define oldF32_TO_F16(x) \
    ((frac16_t) (((uint32_t) x) >> 16))

#define oldF16_TO_F32(x) \
    ((frac32_t) (((uint32_t) x) << 16))
*/
#define F16_TO_F32(x)           \
    ({                          \
        data32_t data;          \
        data.data16.low = 0;    \
        data.data16.frac16 = x; \
        data.frac32;            \
    })

#define F32_TO_F16(x)           \
    ({                          \
        data32_t data;          \
        data.frac32 = x;        \
        data.data16.frac16;     \
    })

#define F16_TO_VAL(x)           \
    ({                          \
        data16_t data;          \
        data.asWord = x;        \
        (uint8_t) (0x80+data.asByte[1]);  \
    })

#define FRAC8(x) \
    ((frac8_t) (127.*(x) + (((x) > 0.0) ? 0.5 : -0.5)))
    
#define FRAC16(x) \
    ((frac16_t) (32768.*(x) + (((x) > 0.0) ? 0.5 : -0.5)))

#define FRAC32(x) \
    ((frac32_t) (2147483648.*(x) + (((x) > 0.0) ? 0.5 : -0.5)))

extern frac32_t fmuls16 (frac16_t, frac16_t);
extern frac32_t fmac16  (frac32_t, frac16_t, frac16_t);
/*
extern point16_t dot_next (wdata_t*);
extern void      dot_init (wire_t*, wdata_t*);
*/

static inline frac16_t _i_get_sin (uint16_t index)
{
    const uint16_t * addr = & sintab[index];

    frac16_t si;

    // p.x = *addr++
    asm ("lpm %A0, Z+\n\tlpm %B0, Z+" : "=&r" (si), "+z" (addr));

    return si;
}

static inline frac16_t _i_get_cos (uint16_t index)
{
    const uint16_t size = sizeof (sintab) / sizeof (sintab[0]) -1;
    const uint16_t * addr = & sintab[index];

    frac16_t co;

    if (index >= 3*size/4)
        addr -= 3*size/4;
    else
        addr += size/4;

    // p.y = *addr++
    asm ("lpm %A0, Z+\n\tlpm %B0, Z+" : "=&r" (co), "+z" (addr));

    return co;
}

static inline frac8_t _i_get_sin8 (uint16_t index)
{
    const uint16_t * addr = & sintab[index];

    frac8_t si;

    // p.x = *addr++
    asm ("lpm %0, Z+\n\tlpm %0, Z+" : "=r" (si), "+z" (addr));

    return si;
}

static inline frac8_t _i_get_cos8 (uint16_t index)
{
    const uint16_t size = sizeof (sintab) / sizeof (sintab[0]) -1;
    const uint16_t * addr = & sintab[index];

    frac16_t co;

    if (index >= 3*size/4)
        addr -= 3*size/4;
    else
        addr += size/4;

    // p.y = *addr++
    asm ("lpm %0, Z+\n\tlpm %0, Z+" : "=r" (co), "+z" (addr));

    return co;
}

extern        point16_t    get_sincos (uint16_t index);
static inline point16_t _i_get_sincos (uint16_t index)
{
    const uint16_t size = sizeof (sintab) / sizeof (sintab[0]) -1;
    const uint16_t * addr = & sintab[index];

    point16_t p;

    // p.x = *addr++
    asm ("lpm %A0, Z+\n\tlpm %B0, Z+" : "=&r" (p), "+z" (addr));

    if (index >= 3*size/4)
        addr -= 3*size/4 +1;
    else
        addr += size/4 -1;

    // p.y = *addr++
    asm ("lpm %C0, Z+\n\tlpm %D0, Z+" : "+r" (p), "+z" (addr));

    return p;
}

extern        point8_t    get_sincos8 (uint16_t index);
static inline point8_t _i_get_sincos8 (uint16_t index)
{
    const uint16_t size = sizeof (sintab) / sizeof (sintab[0]) -1;

    const uint8_t * addr = ((const uint8_t *) & sintab[index]) +1;

    point8_t p;
    
    // p.x = *addr
    asm ("lpm %A0, %a1" : "=&r" (p) : "z" (addr));

    if (index >= 3*size/4)
        addr -= 3*size/2;
    else
        addr += size/2;

    // p.y = *addr
    asm ("lpm %B0, %a1" : "+r" (p) : "z" (addr));

    return p;
}

#if 0
        "sbrc  R0, 7"         "\n\t"      \
        "inc   R1"            "\n\t"      \

#endif

#define fmul8_8(__a,__b)                  \
    ({                                    \
        int8_t __c;                       \
        asm (                             \
        "fmuls %[a], %[b]"    "\n\t"      \
        "mov   %[c], R1"      "\n\t"      \
        "clr    __zero_reg__"             \
        : [c] "=r" (__c)                  \
        : [a] "a" (__a),  [b] "a" (__b)); \
        __c;})
         
#define frotate8(__rx,__ry,               \
                 __px,__py,               \
                 __sincos)                \
        asm (                             \
        "fmuls %[px], %B[sc]"  "\n\t"     \
        "mov   %[rx], R1"      "\n\t"     \
        "fmuls %[py], %A[sc]"  "\n\t"     \
        "add   %[rx], R1"      "\n\t"     \
        "fmuls %[py], %B[sc]"  "\n\t"     \
        "mov   %[ry], R1"      "\n\t"     \
        "fmuls %[px], %A[sc]"  "\n\t"     \
        "sub   %[ry], R1"      "\n\t"     \
        "clr    __zero_reg__"             \
        : [rx] "=&r" (__rx), [ry] "=&r" (__ry) \
        : [px] "a"   (__px), [py] "a"   (__py) \
        , [sc] "a"   (__sincos))
        
#define fmac8_8(__c,__a,__b)              \
        asm (                             \
        "fmuls %[a], %[b]"    "\n\t"      \
        "add   %[c], R1"      "\n\t"      \
        "clr    __zero_reg__"             \
        : [c] "+r" (__c)                  \
        : [a] "a" (__a),  [b] "a" (__b))

#define fmul8_16(__a,__b)                 \
    ({                                    \
        frac16_t __c;                     \
        asm (                             \
        "fmuls %[a], %[b]"    "\n\t"      \
        "movw  %[c], R0"      "\n\t"      \
        "clr    __zero_reg__"             \
        : [c] "=r" (__c)                  \
        : [a] "a" (__a),  [b] "a" (__b)); \
        __c;                              \
    })
        
#define fmac8_16(__c,__a,__b)             \
        asm ("; fmac8_16 %B[c]:%A[c] += %[a] * %[b] \n\t" \
        "fmuls %[a], %[b]"    "\n\t"      \
        "add   %A[c], R0"     "\n\t"      \
        "adc   %B[c], R1"     "\n\t"      \
        "clr    __zero_reg__"             \
        : [c] "+r" (__c)                  \
        : [a] "a" (__a),  [b] "a" (__b))

extern        frac16_t    fmac16_16 (frac16_t c, frac16_t a, frac16_t b);
static inline frac16_t _i_fmac16_16 (frac16_t c, frac16_t a, frac16_t b)
{
    asm ( "; mac16_16 %B[c]:%A[c] += %B[a]:%A[a] * %B[b]:%A[b] \n\t"
        "fmuls  %B[a], %B[b]"       "\n\t"      //; ( (signed)ah * (signed)bh ) << 1
        "movw   "TMP16", R0"        "\n\t"

        "fmul   %A[a], %A[b]"       "\n\t"      //; ( al * bl ) << 1
        "adc    %A[c], "TMP16"+0"   "\n\t"
        "adc    %B[c], "TMP16"+1"   "\n\t"

        "fmulsu %B[a], %A[b]"       "\n\t"      //; ( (signed)ah * bl ) << 1
        "sbc    %B[c], " ZERO       "\n\t"
        "add    %A[c], R1"          "\n\t"
        "adc    %B[c], " ZERO       "\n\t"

        "fmulsu %B[b], %A[a]"       "\n\t"      //; ( (signed)bh * al ) << 1
        "sbc    %B[c], " ZERO       "\n\t"
        "add    %A[c], R1"          "\n\t"
        "adc    %B[c], " ZERO       "\n\t"

        "clr    __zero_reg__"
        : [c] "+r" (c)
        : [a] "a" (a),  [b] "a" (b)
    );

    return c;
}

static inline frac16_t _i_fmul16_16 (frac16_t a, frac16_t b)
{
    frac16_t c;
    asm ( "; fmul16_16 %B[c]:%A[c] = %B[a]:%A[a] * %B[b]:%A[b] \n\t"
        "fmuls  %B[a], %B[b]"       "\n\t"      //; ( (signed)ah * (signed)bh ) << 1
        "movw   %A[c], R0"          "\n\t"

        "fmul   %A[a], %A[b]"       "\n\t"      //; ( al * bl ) << 1
        "adc    %A[c], " ZERO       "\n\t"
        "adc    %B[c], " ZERO       "\n\t"

        "fmulsu %B[a], %A[b]"       "\n\t"      //; ( (signed)ah * bl ) << 1
        "sbc    %B[c], " ZERO       "\n\t"
        "add    %A[c], R1"          "\n\t"
        "adc    %B[c], " ZERO       "\n\t"

        "fmulsu %B[b], %A[a]"       "\n\t"      //; ( (signed)bh * al ) << 1
        "sbc    %B[c], " ZERO       "\n\t"
        "add    %A[c], R1"          "\n\t"
        "adc    %B[c], " ZERO       "\n\t"

        "clr    __zero_reg__"
        : [c] "=&r" (c)
        : [a] "a" (a),  [b] "a" (b)
    );

    return c;
}


//extern wdata_t wdata;

#endif // WIRE_H
